"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var fs = require("fs");
var path = require("path");
var match_star_1 = require("./match-star");
/**
 * Creates a function that can resolve paths according to tsconfig paths property.
 * @param tsConfigPath The paths where tsconfig.json is located.
 * @param baseUrl The baseUrl specified in tsconfig.
 * @param paths The paths specified in tsconfig.
 */
function createMatchPath(tsConfigPath, baseUrl, paths) {
    // Resolve all paths to absolute form once here, this saves time on each request later
    var absoluteBaseUrl = path.dirname(path.join(tsConfigPath, baseUrl));
    var absolutePaths = Object.keys(paths)
        .reduce(function (soFar, key) {
        return (__assign({}, soFar, (_a = {}, _a[key] = paths[key]
            .map(function (pathToResolve) { return path.join(absoluteBaseUrl, pathToResolve); }), _a)));
        var _a;
    }, {});
    return function (sourceFileName, requestedModule, fileExists) {
        return matchFromAbsolutePaths(absolutePaths, sourceFileName, requestedModule, fileExists);
    };
}
exports.createMatchPath = createMatchPath;
/**
 * Finds a path from tsconfig that matches a module load request.
 * @param absolutePaths The paths to try as specified in tsconfig but resolved to absolute form.
 * @param absoluteSourceFileName Absolute path to the file that requested the module.
 * @param requestedModule The required module name.
 * @param fileExists Function that checks for existance of a file (useful for testing).
 * @returns the found path, or undefined if no path was found.
 */
function matchFromAbsolutePaths(absolutePaths, absoluteSourceFileName, requestedModule, fileExists) {
    if (fileExists === void 0) { fileExists = fs.existsSync; }
    if (requestedModule[0] !== '.'
        && requestedModule[0] !== path.sep
        && absolutePaths
        && absoluteSourceFileName
        && requestedModule
        && fileExists) {
        for (var _i = 0, _a = Object.keys(absolutePaths); _i < _a.length; _i++) {
            var key = _a[_i];
            var starMatch = key === requestedModule ? '' : match_star_1.matchStar(key, requestedModule);
            if (starMatch !== undefined) {
                for (var _b = 0, _c = absolutePaths[key]; _b < _c.length; _b++) {
                    var absolutePathToTry = _c[_b];
                    var possibleModule = absolutePathToTry.replace('*', starMatch);
                    if (fileExists(possibleModule)
                        || fileExists(possibleModule + '.ts')
                        || fileExists(possibleModule + '.tsx')) {
                        return possibleModule;
                    }
                }
            }
        }
    }
    return undefined;
}
exports.matchFromAbsolutePaths = matchFromAbsolutePaths;
