"use strict";
var match_path_1 = require("./match-path");
var Tsconfig = require("tsconfig");
/**
 * Installs a custom module load function that can adhere to paths in tsconfig.
 */
function register() {
    // Load tsconfig and create path matching function
    var cwd = process.cwd();
    var loadResult = Tsconfig.loadSync(cwd, undefined);
    if (!loadResult.path) {
        throw new Error("Couldn't find tsconfig");
    }
    var matchPath = match_path_1.createMatchPath(loadResult.path, loadResult.config.compilerOptions.baseUrl, loadResult.config.compilerOptions.paths);
    // Patch node's module loading
    var Module = require('module');
    var originalLoader = Module._load;
    Module._load = function (request, parent) {
        var found = matchPath(parent, request);
        if (found) {
            var modifiedArguments = [found].concat([].slice.call(arguments, 1));
            return originalLoader.apply(this, modifiedArguments);
        }
        return originalLoader.apply(this, arguments);
    };
}
exports.register = register;
